/*
 * EpsilonRemover.cpp
 *
 *  Created on: 16. 1. 2014
 *      Author: tomas
 */

#include "EpsilonRemover.h"

using namespace alib;
using namespace automaton;

namespace epsilon
{

FSM EpsilonRemover::remove( FSM const& origFSM )
{
    FSM fsm;

    for( const auto & state : origFSM.getStates() )
        fsm.addState( state );

    for( const auto & state : origFSM.getInitialStates() )
        fsm.addInitialState( state );

    for( const auto & symbol : origFSM.getInputAlphabet() )
        fsm.addInputSymbol( symbol );

    /**
     * Step 1 from Melichar 2.41
     */
    for( const auto & q : origFSM.getStates( ) )
        for( const auto & p : EpsilonClosure::closure( origFSM, q ) )
            for( const auto & transition : origFSM.getTransitionsFromState( p ) )
                if( transition.getInput() != Symbol( "" ) )
                    fsm.addTransition( q, transition.getInput( ), transition.getTo( ) );

    /**
     * Step 2 from Melichar 2.41
     */
    set<State> finalStates;

    for( const auto & q : fsm.getStates( ) )
    {
        const set<State> & cl = EpsilonClosure::closure( origFSM, q ), & F = origFSM.getFinalStates( );
        set<State> intersect;

        set_intersection( cl.begin(), cl.end(), F.begin(), F.end(), std::inserter( intersect, intersect.begin() ) );
        if( intersect.size( ) != 0 )
            finalStates.insert( q );
    }

    for( const auto & q : finalStates )
        fsm.addFinalState( q );

    return fsm;
}

} /* namespace epsilon */
