/*
 * RandomAutomatonFactory.cpp
 *
 *  Created on: 27. 3. 2014
 *      Author: tomas
 */

#include "RandomAutomatonFactory.h"

using namespace alib;
using namespace alphabet;
using namespace automaton;
using namespace std;

FSM RandomAutomatonFactory::generateNFA( size_t statesCount, set<Symbol> alphabet, double density )
{
    deque<Symbol> alphabet2;
    for( const auto & s : alphabet )
        alphabet2.push_back( s );
    return RandomAutomatonFactory::LeslieConnectedNFA( statesCount, alphabet2, density );
}

FSM RandomAutomatonFactory::generateNFA( size_t statesCount, size_t alphabetSize, double density )
{
    srand( time( NULL ) );

    deque<Symbol> alphabet;
    while( alphabet.size( ) < alphabetSize )
    {
        string s( 1, rand() % 26 + 'a' );
        if( find( alphabet.begin( ), alphabet.end( ), Symbol( s ) ) == alphabet.end( ) )
            alphabet.push_back( Symbol( s ) );
    }

    return RandomAutomatonFactory::LeslieConnectedNFA( statesCount, alphabet, density );
}

FSM RandomAutomatonFactory::LeslieConnectedNFA( size_t n, const deque<Symbol> & alphabet, double density )
{
    if( alphabet.size( ) <= 0 )
        throw AlibException( "Alphabet size must be greater than 0." );


    srand( time( NULL ) );

    FSM automata;

    deque<bool> VStates;
    deque<State> Q;
    int unvisited;


    for( const auto & s : alphabet )
        automata.addInputSymbol( s );

    for( size_t i = 0; i < n; i ++ )
    {
        VStates.push_back( false );
        Q.push_back( State( to_string( i ) ) );
        automata.addState( Q[ i ] );
    }

    if( n == 0 )
    {
        return automata;
    }
    else if( n == 1 )
    {
        automata.addTransition( Q[ 0 ], alphabet[ rand( ) % alphabet.size( ) ], Q[ 0 ] );

        unvisited = 0;
        VStates[ 0 ] = true;
    }
    else
    {
        int x = ( rand( ) % ( n - 1 ) ) + 1;
        automata.addTransition( Q[ 0 ], alphabet[ rand( ) % alphabet.size( ) ], Q[ x ] );
        unvisited = n - 2;

        VStates[ 0 ] = true;
        VStates[ x ] = true;
    }

    automata.addInitialState( Q [ 0 ] );

    while( unvisited != 0 )
    {
        size_t y = rand() % ( n - unvisited ) + 1; // select y-th   accessible state
        size_t z = rand() % unvisited + 1;         // select z-th inaccessible state
        int a = -1, b = -1;

        for( size_t i = 0, cnt = 0; i < n; i++ )
        {
            if( VStates[ i ] == true )
                cnt ++;

            if( cnt == y )
            {
                a = i;
                break;
            }
        }
        for( size_t i = 0, cnt = 0; i < n; i++ )
        {
            if( VStates[ i ] == false )
                cnt ++;

            if( cnt == z )
            {
                b = i;
                break;
            }
        }

        int c = rand() % alphabet.size( );
        automata.addTransition( Q[ a ], alphabet[ c ], Q[ b ] );


        unvisited -= 1;
        VStates[ b ] = true;

        // ---------
    }

    for( State q : Q )
    {
        if( automata.getTransitionsFromState( q ).size( ) == 0 && rand( ) % 100 < 90 )
            automata.addFinalState( q );
        else if( automata.getTransitionsFromState( q ).size( ) > 0 && rand( ) % 100 < 10 )
            automata.addFinalState( q );
    }

    if( n == 1 )
        automata.addFinalState( * automata.getInitialStates( ).begin( ) );

    double mnn100 = 100.0 / alphabet.size( ) / n / n;
    while( automata.getTransitions( ).size( ) * mnn100 < density )
    {
        int y = rand( ) % n;
        int z = rand( ) % n;
        int a = rand( ) % alphabet.size();

        TransitionFSM t ( Q[ y ], alphabet [ a ], Q[ z ] );
        if( ! isInSet( t, automata.getTransitions( ) ) )
            automata.addTransition( t );
    }

    map<Symbol, bool> alphabetUsage;
    for( const auto & a : automata.getInputAlphabet( ) )
        alphabetUsage[ a ] = false;
    for( const auto & t : automata.getTransitions( ) )
        alphabetUsage[ t.getInput( ) ] = true;

    for( const auto & kv : alphabetUsage )
        if( kv.second == false )
            automata.removeInputSymbol( kv.first );

    return automata;
}
