/*
 * arand.cpp
 *
 *  Created on: 26. 3. 2014
 *      Author: tomas
 */

#include <iostream>
#include <cstdlib>
#include <climits>
#include <cfloat>
#include <getopt.h>

#include <automaton/FSM/FSM.h>

#include "RandomAutomatonFactory.h"

using namespace automaton;
using namespace std;

void help( void )
{
    cout << "arand" << endl;
    cout << "Generates random automaton." << endl;
    cout << "Usage: arand -n=N -d=D -a=A" << endl << endl;
    cout << "  -n, --nodes=N \t Number of nodes of automata." << endl;
    cout << "  -d, --density=D \t Percentage of transitions to generate. 1.0 = automata graph is tree, 100.0 = every possible transition." << endl;
    cout << "  -a, --alphabet=A \t Alphabet size." << endl;
}


int main(int argc, char* argv[])
{
    static struct option long_options[] = {
      // common
      {"help",              no_argument,        NULL, 'h'},
//      {"type",              required_argument,  NULL, 't'},

      // automaton options
      {"density",           required_argument,  NULL, 'd'},
      {"nodes",             required_argument,  NULL, 'n'},
      {"alphabet",          required_argument,  NULL, 'a'},

      {0, 0, 0, 0}
    };

    int long_index = 0, opt = 0;

    // string type;
    int statesCount = INT_MAX, alphabetSize = INT_MAX;
    double density = DBL_MAX;

    while( ( opt = getopt_long( argc, argv,"ht:d:n:a:", long_options, & long_index ) )  != -1 )
    {
        switch( opt )
        {
        /*
        case 't':
            type.assign( optarg, strlen( optarg ) );
            break;
         */

        case 'n':
            statesCount = stoi( optarg, nullptr, 10 );
            break;

        case 'd':
            density = stod( optarg, nullptr );
            break;

        case 'a':
            alphabetSize = stoi( optarg, nullptr, 10 );
            break;

        case 'v':
        case 'h':
        default:
            help( );
            return 0;
        }
    }

    if( density < 1 || density > 100 )
    {
        //TODO: floating point arithmetic
        cerr << "You must specify density as a number between 1 and 100." << endl;
        return 1;

    }

    if( statesCount == INT_MAX )
    {
        cerr << "You must specify number of states." << endl;
        return 1;
    }

    if( alphabetSize == INT_MAX )
    {
        cerr << "You must specify alphabet or at least its size." << endl;
        return 1;
    }

    RandomAutomatonFactory::generateNFA( statesCount, alphabetSize, density ).toXML( cout );

    return 0;
}
